#ifndef __Data_Convert_h__
#define __Data_Convert_h__

class DataConv
{
        
    ///****************************//
    //  構造体定義
    ///****************************//

    public:
    /**
     * @union WordByte
     * @brief 2byte(short型)のデータをアクセスできるよう共用体にします
     */
    typedef union
    {
        unsigned char   byte[2];  //!< byte x2
        unsigned short  uint16;   //!< WordByte(short)
    } Uint16Byte;

    typedef union 
    {
        unsigned char   byte[2];  //!< byte x2
        short   int16;   //!< WordByte(short)
    } Int16Byte;


    /**
     * @union DwordByte
     * @brief unsigned 4byte(long型)のデータをアクセスできるよう共用体にします
     */
    typedef union
    {
        unsigned char   byte[4];  //!< byte x4
        unsigned short  uint16[2];  //!< WordByte(short) x2
        unsigned long  uint32;    //!< DoubleWord(long)
    } Uint32Byte;

    /**
     * @union LongByte
     * @brief signed 4byte(long型)のデータをアクセスできるよう共用体にします
     */
    typedef union
    {
        unsigned char   byte[4];  //!< byte x4
        unsigned short  uint16[2];  //!< WordByte(short) x2
        unsigned long  int32;    //!< DoubleWord(long)
    } Int32Byte;

    public:
        static short bytesToInt16(unsigned char byteDatas[]);
        static unsigned short bytesToUint16(unsigned char byteDatas[]);
        static long bytesToInt32(unsigned char byteDatas[]);
        static unsigned long bytesToUint32(unsigned char byteDatas[]);

        static void int16ToBytes(short shortData, unsigned char byteDatas[]);
        static void uint16ToBytes(unsigned short wordData, unsigned char byteDatas[]);
        static void int32ToBytes(long longData, unsigned char byteDatas[]);
        static void uint32ToBytes(unsigned long dwordData, unsigned char byteDatas[]); 


    // /**
    //  * @union QwordByte
    //  * @brief 8byte(long long型)のデータをアクセスできるよう共用体にします
    //  */
    // typedef union
    // {
    // unsigned char   byte[8];  //!< byte x8
    // uint16_t  word[4];  //!< WordByte(short) x4
    // uint32_t  dword[2]; //!< DoubleWord(long) x2
    // uint64_t  qword;    //!< QuadWord(long long)
    // } QwordByte;

    
}; 


#endif