


#include "PmxBaseClass.h"
#include "PmxHardSerialClass.h"
#include <Arduino.h>





/**
 * @brief Construct a new Pmx Hard Serial:: Pmx Hard Serial object
 * 
 * @param hardSerial 
 * @param enPin 
 * @param baudrate 
 * @param timeout 
 */
PmxHardSerial::PmxHardSerial(HardwareSerial *hardSerial,byte enPin,long baudrate, int timeout)
{
    pmxSerial = hardSerial;
    g_timeout = timeout;				//!<	タイムアウトの設定
    g_baudrate = baudrate;			//!<	通信速度の設定
    g_SerialConfig = SERIAL_8N1;
    g_enPin = enPin;                //!<    イネーブルピンの定義

}

/**
 * @brief Construct a new Pmx Hard Serial:: Pmx Hard Serial object
 * 
 * @param hardSerial 
 * @param enPin 
 * @param baudrate 
 * @param serialConfig 
 * @param timeout 
 */
PmxHardSerial::PmxHardSerial(HardwareSerial *hardSerial,byte enPin,long baudrate,unsigned short serialConfig, int timeout)
{
    pmxSerial = hardSerial;
    g_timeout = timeout;				//!<	タイムアウトの設定
    g_baudrate = baudrate;			//!<	通信速度の設定
    g_SerialConfig = serialConfig;
    g_enPin = enPin;                //!<    イネーブルピンの定義
}


/*
PmxHardSerial::PmxHardSerial(HardwareSerial *hardSerial,byte rxPin,byte txPin, byte enPin, long baudrate ,int timeout, byte pmxPartyVal)
{
    pmxSerial = hardSerial;

    g_timeout = timeout;				//!<	タイムアウトの設定
    g_baudrate = baudrate;			//!<	通信速度の設定
    g_party = pmxPartyVal;
    g_enPin = enPin;                //!<    イネーブルピンの定義
    g_rxPin = rxPin;                //!<    RXピンの定義(M5などで必要な場合)
    g_txPin = txPin;                //!<    TXピンの定義(M5などで必要な場合)
}
*/

/**
 * @brief Destroy the Pmx Hard Serial:: Pmx Hard Serial object
 * 
 */
PmxHardSerial::~PmxHardSerial()
{
    if(pmxSerial)
    {
        pmxSerial->end();
    }
}

/**
 * @brief 
 * 
 * @param baudrate 
 * @param timeout 
 * @return true 
 * @return false 
 */
bool PmxHardSerial::begin(long baudrate, int timeout)
{
    if(pmxSerial == nullptr)
    {
        return false;
    }

    

    if(baudrate != PMX::ErrorUint32Data)
    {
        g_baudrate = baudrate;
    }
    if(timeout != PMX::ErrorUint16Data)
    {
        g_timeout = timeout;
    }
    

    pinMode(g_enPin, OUTPUT);

    pmxSerial->begin(g_baudrate,g_SerialConfig);
    //pmxSerial->begin(g_baudrate,SERIAL_8N1,g_rxPin,g_txPin);

    pmxSerial->setTimeout(g_timeout);

    return true;
}

/**
 * @brief 
 * 
 * @return true 
 * @return false 
 */
bool PmxHardSerial::isSynchronize()
{
    return _isSynchronize;
}


/**
 * @brief 
 * 
 * @param txBuf 
 * @param txLen 
 * @param rxBuf 
 * @param rxLen 
 * @return true 
 * @return false 
 */
bool PmxHardSerial::synchronize(byte *txBuf, byte txLen, byte *rxBuf, byte rxLen)
{
    int rxSize; //受信数

	//シリアル初期化確認
	if(pmxSerial == nullptr )
	{
		return false;
	}

    //他の物が通信中だった場合抜ける
    if(_isSynchronize == true)
    {
        return false;
    }

    //通信中にする
    _isSynchronize = true;

    //送信のみをする
    this->__synchronizeWrite(txBuf,txLen);


    //受信データを初期化しておく
    for(int i = 0; i < rxLen; i++)
    {
        receiveBuff[i] = 0xFF;
    }

    //上位のバッファに影響しないように内部の受信バッファに入れておく
	rxSize = pmxSerial->readBytes(receiveBuff, rxLen);
    memcpy(rxBuf, receiveBuff, rxLen);

    //通信中を解除する
    _isSynchronize = false;

	if (rxSize != rxLen) //受信数確認
	{
		return false;
    }

	return true;

	
}


/**
 * @brief 
 * 
 * @param txBuf 
 * @param txLen 
 * @param rxBuf 
 * @param rxLen 
 * @return true 
 * @return false 
 */
bool PmxHardSerial::synchronizeVariableRead(byte *txBuf, byte txLen, byte *rxBuf, byte *rxLen)
{
    //シリアル初期化確認
	if(pmxSerial == nullptr )
	{
		return false;
	}

    //他の物が通信中だった場合抜ける
    if(_isSynchronize == true)
    {
        return false;
    }

    //通信中にする
    _isSynchronize = true;

    //送信のみをする
    this->__synchronizeWrite(txBuf,txLen);

    //受信データを初期化しておく
    for(int i = 0; i < 256; i++)
    {
        receiveBuff[i] = 0xFF;
    }

    //データ数まで受信する
    byte minRxSize = PMX::MinimumLength::Receive - 2 ;//CRCを除く
    byte firstRxBuffSize = pmxSerial->readBytes(receiveBuff, minRxSize);

    //そもそもデータが返ってこなかった
    if(firstRxBuffSize != minRxSize)
    {
        *rxLen = 0;

        //通信中を解除する
        _isSynchronize = false;
        
        return false;
    }

    //　長さを取得する
    byte allRxSize = receiveBuff[PMX::BuffPter::Length];
    byte secondRxBuffSize = allRxSize - minRxSize;

    //上位のバッファに影響しないように内部の受信バッファに入れておく
	byte sBuffSize = pmxSerial->readBytes(&(receiveBuff[minRxSize]), secondRxBuffSize);

    //そもそもデータが返ってこなかった
    if(secondRxBuffSize != sBuffSize)
    {
        *rxLen = minRxSize;

        //通信中を解除する
        _isSynchronize = false;

        return false;
    }

    //受信データを反映させる
    *rxLen = allRxSize;
    //バッファ上のデータをコピーする
    memcpy(rxBuf, receiveBuff, allRxSize);

    //通信中を解除する
    _isSynchronize = false;

	return true;
}

/**
 * @brief 
 * 
 * @param txBuf 
 * @param txLen 
 * @return true 
 * @return false 
 */
bool PmxHardSerial::synchronizeNoRead(byte *txBuf, byte txLen)
{
        //シリアル初期化確認
	if(pmxSerial == nullptr )
	{
		return false;
	}

    //他の物が通信中だった場合抜ける
    if(_isSynchronize == true)
    {
        return false;
    }

    //通信中にする
    _isSynchronize = true;

    //送信のみをする
    this->__synchronizeWrite(txBuf,txLen);

    _isSynchronize = false;

    return true;

}

/**
 * @brief 
 * 
 * @param baudrate 
 * @param parity 
 * @param timeout 
 * @return true 
 * @return false 
 */
// bool PmxHardSerial::setSerialParameters(long baudrate,byte parity,unsigned int timeout)
// {

//     return true;
// }

/**
 * @brief 
 * 
 * @param txBuf 
 * @param txLen 
 */
void PmxHardSerial::__synchronizeWrite(byte *txBuf, byte txLen)
{
    //送信バッファをコピーする
    memcpy(sendBuff, txBuf, txLen);

	pmxSerial->flush(); //待つ

	enHigh(); //送信切替
	pmxSerial->write(sendBuff, txLen);
	pmxSerial->flush();   //待つ
	
	while (pmxSerial->available() > 0) //受信バッファを消す
	{
		// buff = icsSerial->read();	//空読み
		pmxSerial->read();		//空読み
    }

    enLow();  //受信切替
}


/**
 * @brief 
 * 
 * @param outputBytes 
 * @param outputLength 
 */
void PmxHardSerial::logOutputPrint(byte outputBytes[],int outputLength)
{
    if(this->getLogSerial())
    {
        this->getLogSerial()->print("(");
        for(int i = 0; i < outputLength; i++)
        {
            this->getLogSerial()->print("[0x");
            this->getLogSerial()->print(outputBytes[i],HEX);
            this->getLogSerial()->print("]");
        }
        this->getLogSerial()->println(")");
    }
}

